@@ -34,6 +34,7 @@
 #include <vector>
 #include <list>
 #include <memory>
+#include <bits/algorithmfwd.h>
 
 namespace GiNaC {
 
@@ -140,7 +141,7 @@ public:
 
 protected:
 	// helpers
-	static tinfo_t get_tinfo() { return NULL; }
+	static tinfo_t get_tinfo() { return nullptr; }
 	static unsigned get_default_flags() { return 0; }
 	static const char* get_open_delim() { return "("; }
 	static const char* get_close_delim() { return ")"; }
@@ -157,7 +158,7 @@ public:
 			this->seq = s;
 	}
 
-	explicit container(std::auto_ptr<STLT> vp) : inherited(get_tinfo())
+	explicit container(std::unique_ptr<STLT> vp) : inherited(get_tinfo())
 	{
 		setflag(get_default_flags());
 		this->seq.swap(*vp);
@@ -361,33 +362,34 @@ public:
 
 	// functions overriding virtual functions from base classes
 public:
-	bool info(unsigned inf) const { return inherited::info(inf); }
-	unsigned precedence() const { return 10; }
-	size_t nops() const { return this->seq.size(); }
-	ex op(size_t i) const;
-	ex & let_op(size_t i);
-	ex eval(int level = 0) const;
-	ex subs(const exmap & m, unsigned options = 0) const;
+	bool info(unsigned inf) const override { return inherited::info(inf); }
+	unsigned precedence() const override { return 10; }
+	size_t nops() const override { return this->seq.size(); }
+	const ex op(size_t i) const override;
+	ex & let_op(size_t i) override;
+	ex eval(int level = 0) const override;
+	ex subs(const exmap & m, unsigned options = 0) const override;
 
 protected:
-	ex conjugate() const
+	ex conjugate() const override
 	{
-		STLT *newcont = NULL;
-		for (const_iterator i=this->seq.begin(); i!=this->seq.end(); ++i) {
+		STLT *newcont = nullptr;
+		for (const auto & elem : this->seq) {
+			const ex conj = elem.conjugate();
 			if (newcont) {
-				newcont->push_back(i->conjugate());
+				newcont->push_back(conj);
 				continue;
 			}
-			ex x = i->conjugate();
-			if (are_ex_trivially_equal(x, *i)) {
+			if (are_ex_trivially_equal(conj, elem)) {
 				continue;
 			}
 			newcont = new STLT;
 			this->reserve(*newcont, this->seq.size());
-			for (const_iterator j=this->seq.begin(); j!=i; ++j) {
-				newcont->push_back(*j);
+			for (const auto & elem2 : this->seq) {
+				if (&elem2 != &elem)
+					newcont->push_back(elem2);
 			}
-			newcont->push_back(x);
+			newcont->push_back(conj);
 		}
 		if (newcont) {
 			ex result = thiscontainer(*newcont);
@@ -397,29 +399,25 @@ protected:
 		return *this;
 	}
 
-	ex real_part() const
+	ex real_part() const override
 	{
 		STLT cont;
 		this->reserve(cont, nops());
-		const_iterator b = begin();
-		const_iterator e = end();
-		for(const_iterator i=b; i!=e; ++i)
-			cont.push_back(i->real_part());
+		for (const auto & elem : this->seq)
+			cont.push_back(elem.real_part());
 		return thiscontainer(cont);
 	}
 
-	ex imag_part() const
+	ex imag_part() const override
 	{
 		STLT cont;
 		this->reserve(cont, nops());
-		const_iterator b = begin();
-		const_iterator e = end();
-		for(const_iterator i=b; i!=e; ++i)
-			cont.push_back(i->imag_part());
+		for (const auto & elem : this->seq)
+			cont.push_back(elem.imag_part());
 		return thiscontainer(cont);
 	}
 
-	bool is_equal_same_type(const basic & other) const;
+	bool is_equal_same_type(const basic & other) const override;
 
 	// new virtual functions which can be overridden by derived classes
 protected:
@@ -429,7 +427,7 @@ protected:
 
 	/** Similar to duplicate(), but with a preset sequence (which gets
 	 *  deleted). Must be overridden by derived classes. */
-	virtual ex thiscontainer(std::auto_ptr<STLT> vp) const { return container(vp); }
+	virtual ex thiscontainer(std::unique_ptr<STLT> vp) const { return container(std::move(vp)); }
 
 	virtual void printseq(const print_context & c, const char* openbracket,
 			char delim, const char* closebracket,
@@ -445,7 +443,7 @@ private:
 
 	void unique_()
 	{
-		typename STLT::iterator p = std::unique(this->seq.begin(), this->seq.end(), ex_is_equal());
+		auto p = std::unique(this->seq.begin(), this->seq.end(), ex_is_equal());
 		this->seq.erase(p, this->seq.end());
 	}
 
@@ -464,12 +462,12 @@ public:
 	const_reverse_iterator rend() const {return this->seq.rend();}
 
 protected:
-	void do_print(const print_context & c, unsigned level) const;
-	void do_print_tree(const print_tree & c, unsigned level) const;
+	void do_print(const print_context & c, unsigned level) const override;
+	void do_print_tree(const print_tree & c, unsigned level) const override;
 	void do_print_python(const print_python & c, unsigned level) const;
-	void do_print_python_repr(const print_python_repr & c, unsigned level) const;
+	void do_print_python_repr(const print_python_repr & c, unsigned level) const override;
 	STLT evalchildren(int level) const;
-	std::auto_ptr<STLT> subschildren(const exmap & m, unsigned options = 0) const;
+	std::unique_ptr<STLT> subschildren(const exmap & m, unsigned options = 0) const;
 };
 
 /** Default constructor */
@@ -485,11 +483,11 @@ container<C>::container(const archive_node &n, lst &sym_lst) : inherited(n, sym_
 {
 	setflag(get_default_flags());
 
-	archive_node::archive_node_cit first = n.find_first("seq");
-	archive_node::archive_node_cit last = n.find_last("seq");
+	auto first = n.find_first("seq");
+	auto last = n.find_last("seq");
 	++last;
 	this->reserve(this->seq, last - first);
-	for (archive_node::archive_node_cit i=first; i<last; ++i) {
+	for (auto i=first; i<last; ++i) {
 		ex e;
 		n.find_ex_by_loc(i, e, sym_lst);
 		this->seq.push_back(e);
@@ -508,15 +506,12 @@ template <template <class T, class = std::allocator<T> > class C>
 void container<C>::archive(archive_node &n) const
 {
 	inherited::archive(n);
-	const_iterator i = this->seq.begin(), end = this->seq.end();
-	while (i != end) {
-		n.add_ex("seq", *i);
-		++i;
-	}
+	for (const auto & elem : this->seq)
+		n.add_ex("seq", elem);
 }
 
 template <template <class T, class = std::allocator<T> > class C>
-void container<C>::do_print(const print_context & c, unsigned level) const
+void container<C>::do_print(const print_context & c, unsigned) const
 {
 	// always print brackets around seq, ignore upper_precedence
 	printseq(c, get_open_delim(), ',', get_close_delim(), precedence(), precedence()+1);
@@ -529,10 +524,8 @@ void container<C>::do_print_tree(const print_tree & c, unsigned level) const
 	    << std::hex << ", hash=0x" << hashvalue << ", flags=0x" << flags << std::dec
 	    << ", nops=" << nops()
 	    << std::endl;
-	const_iterator i = this->seq.begin(), end = this->seq.end();
-	while (i != end) {
-		i->print(c, level + c.delta_indent);
-		++i;
+	for (const auto & elem : this->seq) {
+		elem.print(c, level + c.delta_indent);
 	}
 	c.s << std::string(level + c.delta_indent,' ') << "=====" << std::endl;
 }
@@ -544,18 +537,18 @@ void container<C>::do_print_python(const print_python & c, unsigned level) const
 }
 
 template <template <class T, class = std::allocator<T> > class C>
-void container<C>::do_print_python_repr(const print_python_repr & c, unsigned level) const
+void container<C>::do_print_python_repr(const print_python_repr & c, unsigned) const
 {
 	c.s << class_name();
 	printseq(c, "(", ',', ")", precedence(), precedence()+1);
 }
 
 template <template <class T, class = std::allocator<T> > class C>
-ex container<C>::op(size_t i) const
+const ex container<C>::op(size_t i) const
 {
 	GINAC_ASSERT(i < nops());
 
-	const_iterator it = this->seq.begin();
+	auto it = this->seq.begin();
 	advance(it, i);
 	return *it;
 }
@@ -566,7 +559,7 @@ ex & container<C>::let_op(size_t i)
 	GINAC_ASSERT(i < nops());
 
 	ensure_if_modifiable();
-	typename STLT::iterator it = this->seq.begin();
+	auto it = this->seq.begin();
 	advance(it, i);
 	return *it;
 }
@@ -591,9 +584,9 @@ ex container<C>::subs(const exmap & m, unsigned options) const
 	// f(x).subs(x==f^-1(x))
 	//   -> f(f^-1(x))  [subschildren]
 	//   -> x           [eval]   /* must not subs(x==f^-1(x))! */
-	std::auto_ptr<STLT> vp = subschildren(m, options);
+	std::unique_ptr<STLT> vp = subschildren(m, options);
 	if (vp.get()) {
-		ex result(thiscontainer(vp));
+		ex result(thiscontainer(std::move(vp)));
 		if (is_a<container<C> >(result))
 			return ex_to<basic>(result).subs_one_level(m, options);
 		else
@@ -613,7 +606,7 @@ int container<C>::compare_same_type(const basic & other) const
 	GINAC_ASSERT(is_a<container>(other));
 	const container & o = static_cast<const container &>(other);
 
-	const_iterator it1 = this->seq.begin(), it1end = this->seq.end(),
+	auto it1 = this->seq.begin(), it1end = this->seq.end(),
 	               it2 = o.seq.begin(), it2end = o.seq.end();
 
 	while (it1 != it1end && it2 != it2end) {
@@ -635,14 +628,14 @@ bool container<C>::is_equal_same_type(const basic & other) const
 	if (this->seq.size() != o.seq.size())
 		return false;
 
-	const_iterator it1 = this->seq.begin(), it1end = this->seq.end(), it2 = o.seq.begin();
-	while (it1 != it1end) {
-		if (!it1->is_equal(*it2))
-			return false;
-		++it1; ++it2;
-	}
+        auto it1 = this->seq.begin(), it1end = this->seq.end(), it2 = o.seq.begin();
+        while (it1 != it1end) {
+                if (!it1->is_equal(*it2))
+                        return false;
+                ++it1; ++it2;
+       }
 
-	return true;
+       return true;
 }
 
 /** Add element at front. */
@@ -746,7 +739,7 @@ void container<C>::printseq(const print_context & c, const char* openbracket,
 		c.s << openbracket;
 
 	if (!this->seq.empty()) {
-		const_iterator it = this->seq.begin(), itend = this->seq.end();
+		auto it = this->seq.begin(), itend = this->seq.end();
 		--itend;
 		while (it != itend) {
 			it->print(c, this_precedence);
@@ -772,29 +765,27 @@ typename container<C>::STLT container<C>::evalchildren(int level) const
 	this->reserve(s, this->seq.size());
 
 	--level;
-	const_iterator it = this->seq.begin(), itend = this->seq.end();
-	while (it != itend) {
-		s.push_back(it->eval(level));
-		++it;
+	for (const auto & elem : this->seq) {
+		s.push_back(elem.eval(level));
 	}
 
 	return s;
 }
 
 template <template <class T, class = std::allocator<T> > class C>
-std::auto_ptr<typename container<C>::STLT> container<C>::subschildren(const exmap & m, unsigned options) const
+std::unique_ptr<typename container<C>::STLT> container<C>::subschildren(const exmap & m, unsigned options) const
 {
 	// returns a NULL pointer if nothing had to be substituted
 	// returns a pointer to a newly created STLT otherwise
 	// (and relinquishes responsibility for the STLT)
 
-	const_iterator cit = this->seq.begin(), end = this->seq.end();
-	while (cit != end) {
+	auto cit = this->seq.begin(), cend = this->seq.end();
+	while (cit != cend) {
 		const ex & subsed_ex = cit->subs(m, options);
 		if (!are_ex_trivially_equal(*cit, subsed_ex)) {
 
 			// copy first part of seq which hasn't changed
-			std::auto_ptr<STLT> s(new STLT(this->seq.begin(), cit));
+			std::unique_ptr<STLT> s(new STLT(this->seq.begin(), cit));
 			this->reserve(*s, this->seq.size());
 
 			// insert changed element
@@ -802,18 +793,18 @@ std::auto_ptr<typename container<C>::STLT> container<C>::subschildren(const exma
 			++cit;
 
 			// copy rest
-			while (cit != end) {
+			while (cit != cend) {
 				s->push_back(cit->subs(m, options));
 				++cit;
 			}
 
-			return s;
+			return std::move(s);
 		}
 
 		++cit;
 	}
 	
-	return std::auto_ptr<STLT>(0); // nothing has changed
+	return std::unique_ptr<STLT>(nullptr); // nothing has changed
 }
 
 } // namespace GiNaC
